#!/usr/bin/python
# -*- coding: utf-8 -*-

# Libervia: a Salut à Toi frontend
# Copyright (C) 2011, 2012, 2013, 2014 Jérôme Poisson <goffi@goffi.org>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" Contacts / jids related panels """

import pyjd  # this is dummy in pyjs
from sat.core.log import getLogger
log = getLogger(__name__)
from sat_frontends.tools import jid

from pyjamas.ui.VerticalPanel import VerticalPanel

import html_tools
import contact_widget
from constants import Const as C


class ContactsPanel(VerticalPanel):
    """ContactList graphic representation

    Special features like popup menu panel or changing the contact states must be done in a sub-class.
    """

    def __init__(self, host, merge_resources=True, contacts_click=None,
                 contacts_style=None, contacts_menus=True,
                 contacts_display=C.CONTACT_DEFAULT_DISPLAY):
        """

        @param host (SatWebFrontend): host instance
        @param merge_resources (bool): if True, the entities sharing the same
            bare JID will also share the same contact box.
        @param contacts_click (callable): click callback for the contact boxes
        @param contacts_style (unicode): CSS style name for the contact boxes
        @param contacts_menus (tuple): define the menu types that fit this
            contact panel, with values from the menus type constants.
        @param contacts_display (tuple): prioritize the display methods of the
            contact's label with values in ("jid", "nick", "bare", "resource")
        """
        VerticalPanel.__init__(self)
        self.host = host
        self.merge_resources = merge_resources
        self._contacts = {}  # entity jid to ContactBox map
        self.click_listener = None

        if contacts_click is not None:
            self.onClick = contacts_click

        self.contacts_style = contacts_style
        self.contacts_menus = contacts_menus
        self.contacts_display = contacts_display

    def _key(self, contact_jid):
        """Return internal key for this contact.

        @param contact_jid (jid.JID): contact JID
        @return: jid.JID
        """
        return contact_jid.bare if self.merge_resources else contact_jid

    def clear(self):
        """Clear all contacts."""
        self._contacts.clear()
        VerticalPanel.clear(self)

    def setList(self, jids):
        """set all contacts in the list in one shot.

        @param jids (list[jid.JID]): jids to display (the order is kept)
        @param name (unicode): optional name of the contact
        """
        # FIXME: we do a full clear and add boxes after, we should only remove recently hidden boxes and add new ones, and re-order
        current = [box.jid for box in self.children if isinstance(box, contact_widget.ContactBox)]
        if current == jids:
            # the display doesn't change
            return
        self.clear()
        for contact_jid in jids:
            assert isinstance(contact_jid, jid.JID)
            self.addContact(contact_jid)

    def getContactBox(self, contact_jid):
        """Get a contact box for a contact, add it if it doesn't exist yet.

        @param contact_jid (jid.JID): contact JID
        @return: ContactBox
        """
        try:
            return self._contacts[self._key(contact_jid)]
        except KeyError:
            box = contact_widget.ContactBox(self.host, contact_jid,
                                            style_name=self.contacts_style,
                                            display=self.contacts_display,
                                            plugin_menu_context=self.contacts_menus)
            self._contacts[self._key(contact_jid)] = box
            return box

    def addContact(self, contact_jid):
        """Add a contact to the list.

        @param contact_jid (jid.JID): contact JID
        """
        box = self.getContactBox(contact_jid)
        if box not in self.children:
            VerticalPanel.append(self, box)

    def removeContact(self, contact_jid):
        """Remove a contact from the list.

        @param contact_jid (jid.JID): contact JID
        """
        box = self._contacts.pop(self._key(contact_jid))
        VerticalPanel.remove(self, box)

    def updateAvatar(self, contact_jid, url):
        """Update the avatar of the given contact

        @param contact_jid (jid.JID): contact JID
        @param url (unicode): image url
        """
        self.getContactBox(contact_jid).updateAvatar(url)

    def updateNick(self, contact_jid, new_nick):
        """Update the avatar of the given contact.

        @param contact_jid (jid.JID): contact JID
        @param new_nick (unicode): new nick of the contact
        """
        self.getContactBox(contact_jid).updateNick(new_nick)

    def setPresence(self, entity, show):
        """Update entity's presence.

        @param entity(jid.JID): entity updated
        @param show: availability
        """
        if self.merge_resources:  # we use cache to have the show information of main resource only
            clist = self.host.contact_list
            show = clist.getCache(entity.bare, C.PRESENCE_SHOW)
            if show is None:
                show = C.PRESENCE_UNAVAILABLE
        html_tools.setPresenceStyle(self.getContactBox(entity).label, show)
